package api

import (
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

//会员
func Member(r *GinRouter) {

	// 查看会员
	r.GETJWTKEY("/member/detail/:id", GetMember, SKU3DADMIN)

	// 查看会员列表
	r.GETJWT("/member/list", GetMembers)

	// 添加会员
	r.POSTJWTKEY("/member/create", CreateMember, SKU3DADMIN)

	// 修改会员
	r.POSTJWTKEY("/member/update", UpdateMember, SKU3DADMIN)

	// 删除会员
	r.POSTJWTKEY("/member/delete/:id", DeleteMember, SKU3DADMIN)

}

func GetMember(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	idStr := c.Param("id")
	id, err := primitive.ObjectIDFromHex(idStr)
	if err != nil {
		return false, err
	}

	var member model.Member

	option := &repo.DocSearchOptions{
		CollectName: repo.CollectionMember,
		Query:       repo.Map{"_id": id},
	}

	found, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), option, &member)
	if !found {
		return false, err
	}

	return member, nil
}

func GetMembers(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	page, size, query := UtilQueryPageSize(c)

	option := &repo.PageSearchOptions{
		CollectName: repo.CollectionMember,
		Query:       query,
		Page:        page,
		Size:        size,
		Sort:        bson.M{"createTime": -1},
	}

	return repo.RepoPageSearch(apictx.CreateRepoCtx(), option)
}

func CreateMember(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	var member model.Member
	err := c.ShouldBindJSON(&member)
	if err != nil {
		return false, nil
	}

	ctx := apictx.CreateRepoCtx()

	// option := &repo.DocSearchOptions{
	// 	CollectName: repo.CollectionMember,
	// 	Query: repo.Map{"key":member.Key},
	// }
	// found, _ := repo.RepoSeachDocMap(ctx, option)
	// if found{
	// 	return false,NewError("已存在key")
	// }

	member.CreateTime = time.Now()
	member.UpdateTime = time.Now()

	return repo.RepoAddDoc(ctx, repo.CollectionMember, &member)
}

func UpdateMember(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	var member model.Member
	err := c.ShouldBindJSON(&member)
	if err != nil {
		return false, nil
	}

	member.UpdateTime = time.Now()

	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionMember, member.Id.Hex(), &member)
}

func DeleteMember(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	idStr := c.Param("id")

	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionMember, idStr)
}
